/*!
 * @file resource_manager.h
 */

#ifndef _RESOURCE_MANAGER_H
#define _RESOURCE_MANAGER_H

#include "resource.h"
#include "filesys/file.h"
#include "filesys/directory.h"

namespace Resources
{

  class ResourceManager : public BaseObject
  {
    ObjectListDeclaration(ResourceManager);
  public:
    /** @returns a Pointer to the only object of this Class */
    inline static ResourceManager* getInstance() { if (!_singletonRef) _singletonRef = new ResourceManager();  return _singletonRef; };
    /** @brief deletes the Instance if it exists. */
    inline static void deleteInstance() { if (_singletonRef) delete _singletonRef; };


    void setMainGlobalPath(const Directory& directory);
    void addGlobalPath(const Directory& directory);

    bool addResourcePath(const std::string& resourceName, const std::string& pathName);
    bool addResourceSubPath(const std::string& resourceName, const std::string& pathName);
    void registerType(Resources::Type* type);
    void unregisterType(Resources::Type* type);

    unsigned int addKeepLevelName(const std::string& keepLevelName);
    unsigned int getKeepLevelID(const std::string& keepLevelName) const;
    const std::string& getKeepLevelName(unsigned int keepLevelID) const;

    /** @returns the Types of Resources */
    const std::vector<Resources::Type*> resourceTypes() const { return _resourceTypes; };
    /** @returns the main global search Path */
    const Directory& mainGlobalPath() const { return _mainGlobalPath; };
    /** @returns all global paths without mainGlobalPath */
    const std::vector<Directory>& globalPaths() const { return _globalPaths; };


    bool checkFileInMainPath(const File& fileInside);
    std::string prependAbsoluteMainPath(const std::string& fileName);

    void unloadAllBelowKeepLevel(const Resources::KeepLevel& keepLevel);

    void debug() const;

    // utility functions for handling files in and around the data-directory
    static std::string getFullName(const std::string& fileName);
    static bool isInDataDir(const std::string& fileName);
  private:
    ResourceManager();
    virtual ~ResourceManager();

  private:
    static ResourceManager*         _singletonRef;       //!< singleton Reference

    Directory                          _mainGlobalPath;     //!< The main include directory (default at "./")
    std::vector<Directory>             _globalPaths;        //!< Additional Global include directories.

    std::vector<Resources::Type*>      _resourceTypes;      //!< A Vector of all the stored ResourceTypes @see Resources::Type
    std::vector<std::string>           _keepLevelNames;     //!< Names of KeepLevels @see Resources::KeepLevel
  };

}

#endif /* _RESOURCE_MANAGER_H */
